//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8UB2_Register_Enums.h>                  //SFR declarations
#include "InitDevice.h"
#include "VCPXpress.h"
#include "descriptor.h"
#include "Project_DefsVarsFuncs.h"

//-----------------------------------------------------------------------------
// main() routine
// ----------------------------------------------------------------------------
int main(void)
{
	float Measured_Temp;
	float Error, Previous_Error, Error_Integral, Error_Derivative;
	unsigned char K_proportional, K_integral, K_derivative;
	float PID_Output;
	unsigned char Heater_Drive;

	//call hardware initialization routine
	enter_DefaultMode_from_RESET();

	//ensure that MAX31855 slave select is inactive
	TC_NSS = HIGH;

	//ensure that DAC slave select is inactive
	DAC_NSS = HIGH;

	//set heater-drive voltage to 0 V
	UpdateDAC(DAC_HEATER, 0);

	//VCPXpress initialization
	USB_Init(&InitStruct);

	//enable VCPXpress API interrupts
	API_Callback_Enable(myAPICallback);

	//enable global interrupts
	IE_EA = 1;

	//refer to the PCA ISR for information on the delay functionality
	PCA0 = 0x0000 - PID_INTERVAL;
	PCA0CN0_CR = PCA_RUN;

	Setpoint_Temp = 50;

	K_proportional = 40;
	K_integral = 10;
	K_derivative = 0;

	Error_Integral = 0;
	Previous_Error = 0;

	/*Apparently the MAX31855 generates better
	 * temperature data if it has a little extra time
	 * after power-up. This is why we have a 1-second
	 * delay here.*/
	Delay_10ms(100);

	while (1)
	{
		/*First, we check PID_ACTIVE.The following while statement
		 * suspends PID functionality until the EFM8 is commanded by
		 * Scilab to begin or resume PID control.*/
		while(PID_ACTIVE == FALSE);

		GatherMAX31855Data();
		while(TEMP_DATA_READY == FALSE);	//wait until the SPI transaction is complete

		Measured_Temp = ConvertMAX31855Data_to_TempC();

		//send measured temperature to Scilab
		TransmitUSB_TempData();

		Error = Setpoint_Temp - Measured_Temp;

		/*We don't want the integral error to get
		 * way too large. This is a standard problem
		 * referred to as integral windup. One solution
		 * is to simply restrict the integral error to
		 * reasonable values.*/
		Error_Integral = Error_Integral + Error;
		if(Error_Integral > 50)
			Error_Integral = 50;
		else if(Error_Integral < -50)
			Error_Integral = -50;

		Error_Derivative = Error - Previous_Error;
		Previous_Error = Error;

		PID_Output = (K_proportional*Error) + (K_integral*Error_Integral) + (K_derivative*Error_Derivative);

		/*We need to restrict the PID output to
		 * acceptable values. Here we have limited it
		 * to a maximum of 200, which corresponds
		 * to about 780 mA of heater-drive current, and
		 * a minimum of 0, because we cannot drive
		 * less than 0 A through the heating
		 * element.*/
		if(PID_Output > 200)
			PID_Output = 200;
		else if(PID_Output < 0)
			PID_Output = 0;

		//here we convert the PID output from a float to an unsigned char
		Heater_Drive = PID_Output;

		UpdateDAC(DAC_HEATER, Heater_Drive);

		/*LED visualization: If the measured temperature is within
		 * plus/minus 2 degrees C of the setpoint, we turn on the
		 * green LED. If the measured temperature is more than
		 * 2 degrees below the setpoint, we turn on the blue LED.
		 * If the measured temperature is more than 2 degrees
		 * above the setpoint, we turn on the red LED.*/
		if(Measured_Temp >= (Setpoint_Temp-2) && Measured_Temp <= (Setpoint_Temp+2))
		{
			UpdateDAC(DAC_RGB_R, 0);
			UpdateDAC(DAC_RGB_B, 0);
			UpdateDAC(DAC_RGB_G, 100);
		}

		else if(Measured_Temp < (Setpoint_Temp-2))
		{
			UpdateDAC(DAC_RGB_R, 0);
			UpdateDAC(DAC_RGB_B, 100);
			UpdateDAC(DAC_RGB_G, 0);
		}

		else if(Measured_Temp > (Setpoint_Temp+2))
		{
			UpdateDAC(DAC_RGB_R, 100);
			UpdateDAC(DAC_RGB_B, 0);
			UpdateDAC(DAC_RGB_G, 0);
		}

		/*Here we wait until the PID interval has expired,
		 * then we begin a new iteration. The interval is
		 * currently set to 2 seconds.*/
		PID_WAIT = TRUE;
		while(PID_WAIT == TRUE);
	}
}
